import { each, extend, includes, registerEvent } from './utils';
import { autocaptureCompatibleElements, getClassNames, getDirectAndNestedSpanText, getElementsChainString, getSafeText, isAngularStyleAttr, isDocumentFragment, isElementNode, isSensitiveElement, isTag, isTextNode, makeSafeText, shouldCaptureDomEvent, shouldCaptureElement, shouldCaptureValue, splitClassString, } from './autocapture-utils';
import RageClick from './extensions/rageclick';
import { AUTOCAPTURE_DISABLED_SERVER_SIDE } from './constants';
import { isBoolean, isFunction, isNull, isObject, isUndefined } from './utils/type-utils';
import { logger } from './utils/logger';
import { document, window } from './utils/globals';
var COPY_AUTOCAPTURE_EVENT = '$copy_autocapture';
function limitText(length, text) {
    if (text.length > length) {
        return text.slice(0, length) + '...';
    }
    return text;
}
var Autocapture = /** @class */ (function () {
    function Autocapture(instance) {
        this._initialized = false;
        this._isDisabledServerSide = null;
        this.rageclicks = new RageClick();
        this._elementsChainAsString = false;
        this.instance = instance;
    }
    Object.defineProperty(Autocapture.prototype, "config", {
        get: function () {
            var _a;
            var config = isObject(this.instance.config.autocapture) ? this.instance.config.autocapture : {};
            // precompile the regex
            config.url_allowlist = (_a = config.url_allowlist) === null || _a === void 0 ? void 0 : _a.map(function (url) { return new RegExp(url); });
            return config;
        },
        enumerable: false,
        configurable: true
    });
    Autocapture.prototype._addDomEventHandlers = function () {
        var _this = this;
        if (!this.isBrowserSupported()) {
            logger.info('Disabling Automatic Event Collection because this browser is not supported');
            return;
        }
        if (!window || !document) {
            return;
        }
        var handler = function (e) {
            e = e || (window === null || window === void 0 ? void 0 : window.event);
            try {
                _this._captureEvent(e);
            }
            catch (error) {
                logger.error('Failed to capture event', error);
            }
        };
        var copiedTextHandler = function (e) {
            e = e || (window === null || window === void 0 ? void 0 : window.event);
            _this._captureEvent(e, COPY_AUTOCAPTURE_EVENT);
        };
        registerEvent(document, 'submit', handler, false, true);
        registerEvent(document, 'change', handler, false, true);
        registerEvent(document, 'click', handler, false, true);
        if (this.config.capture_copied_text) {
            registerEvent(document, 'copy', copiedTextHandler, false, true);
            registerEvent(document, 'cut', copiedTextHandler, false, true);
        }
    };
    Autocapture.prototype.startIfEnabled = function () {
        if (this.isEnabled && !this._initialized) {
            this._addDomEventHandlers();
            this._initialized = true;
        }
    };
    Autocapture.prototype.afterDecideResponse = function (response) {
        var _a;
        if (response.elementsChainAsString) {
            this._elementsChainAsString = response.elementsChainAsString;
        }
        if (this.instance.persistence) {
            this.instance.persistence.register((_a = {},
                _a[AUTOCAPTURE_DISABLED_SERVER_SIDE] = !!response['autocapture_opt_out'],
                _a));
        }
        // store this in-memory in case persistence is disabled
        this._isDisabledServerSide = !!response['autocapture_opt_out'];
        this.startIfEnabled();
    };
    Object.defineProperty(Autocapture.prototype, "isEnabled", {
        get: function () {
            var _a, _b;
            var persistedServerDisabled = (_a = this.instance.persistence) === null || _a === void 0 ? void 0 : _a.props[AUTOCAPTURE_DISABLED_SERVER_SIDE];
            var memoryDisabled = this._isDisabledServerSide;
            if (isNull(memoryDisabled) &&
                !isBoolean(persistedServerDisabled) &&
                !this.instance.config.advanced_disable_decide) {
                // We only enable if we know that the server has not disabled it (unless decide is disabled)
                return false;
            }
            var disabledServer = (_b = this._isDisabledServerSide) !== null && _b !== void 0 ? _b : !!persistedServerDisabled;
            var disabledClient = !this.instance.config.autocapture;
            return !disabledClient && !disabledServer;
        },
        enumerable: false,
        configurable: true
    });
    Autocapture.prototype._previousElementSibling = function (el) {
        if (el.previousElementSibling) {
            return el.previousElementSibling;
        }
        var _el = el;
        do {
            _el = _el.previousSibling; // resolves to ChildNode->Node, which is Element's parent class
        } while (_el && !isElementNode(_el));
        return _el;
    };
    Autocapture.prototype._getAugmentPropertiesFromElement = function (elem) {
        var shouldCaptureEl = shouldCaptureElement(elem);
        if (!shouldCaptureEl) {
            return {};
        }
        var props = {};
        each(elem.attributes, function (attr) {
            if (attr.name && attr.name.indexOf('data-ph-capture-attribute') === 0) {
                var propertyKey = attr.name.replace('data-ph-capture-attribute-', '');
                var propertyValue = attr.value;
                if (propertyKey && propertyValue && shouldCaptureValue(propertyValue)) {
                    props[propertyKey] = propertyValue;
                }
            }
        });
        return props;
    };
    Autocapture.prototype._getPropertiesFromElement = function (elem, maskInputs, maskText) {
        var _a;
        var tag_name = elem.tagName.toLowerCase();
        var props = {
            tag_name: tag_name,
        };
        if (autocaptureCompatibleElements.indexOf(tag_name) > -1 && !maskText) {
            if (tag_name.toLowerCase() === 'a' || tag_name.toLowerCase() === 'button') {
                props['$el_text'] = limitText(1024, getDirectAndNestedSpanText(elem));
            }
            else {
                props['$el_text'] = limitText(1024, getSafeText(elem));
            }
        }
        var classes = getClassNames(elem);
        if (classes.length > 0)
            props['classes'] = classes.filter(function (c) {
                return c !== '';
            });
        // capture the deny list here because this not-a-class class makes it tricky to use this.config in the function below
        var elementAttributeIgnorelist = (_a = this.config) === null || _a === void 0 ? void 0 : _a.element_attribute_ignorelist;
        each(elem.attributes, function (attr) {
            // Only capture attributes we know are safe
            if (isSensitiveElement(elem) && ['name', 'id', 'class', 'aria-label'].indexOf(attr.name) === -1)
                return;
            if (elementAttributeIgnorelist === null || elementAttributeIgnorelist === void 0 ? void 0 : elementAttributeIgnorelist.includes(attr.name))
                return;
            if (!maskInputs && shouldCaptureValue(attr.value) && !isAngularStyleAttr(attr.name)) {
                var value = attr.value;
                if (attr.name === 'class') {
                    // html attributes can _technically_ contain linebreaks,
                    // but we're very intolerant of them in the class string,
                    // so we strip them.
                    value = splitClassString(value).join(' ');
                }
                props['attr__' + attr.name] = limitText(1024, value);
            }
        });
        var nthChild = 1;
        var nthOfType = 1;
        var currentElem = elem;
        while ((currentElem = this._previousElementSibling(currentElem))) {
            // eslint-disable-line no-cond-assign
            nthChild++;
            if (currentElem.tagName === elem.tagName) {
                nthOfType++;
            }
        }
        props['nth_child'] = nthChild;
        props['nth_of_type'] = nthOfType;
        return props;
    };
    Autocapture.prototype._getDefaultProperties = function (eventType) {
        return {
            $event_type: eventType,
            $ce_version: 1,
        };
    };
    Autocapture.prototype._getEventTarget = function (e) {
        var _a;
        // https://developer.mozilla.org/en-US/docs/Web/API/Event/target#Compatibility_notes
        if (isUndefined(e.target)) {
            return e.srcElement || null;
        }
        else {
            if ((_a = e.target) === null || _a === void 0 ? void 0 : _a.shadowRoot) {
                return e.composedPath()[0] || null;
            }
            return e.target || null;
        }
    };
    Autocapture.prototype._captureEvent = function (e, eventName) {
        var _this = this;
        var _a, _b, _c, _d;
        if (eventName === void 0) { eventName = '$autocapture'; }
        if (!this.isEnabled) {
            return;
        }
        /*** Don't mess with this code without running IE8 tests on it ***/
        var target = this._getEventTarget(e);
        if (isTextNode(target)) {
            // defeat Safari bug (see: http://www.quirksmode.org/js/events_properties.html)
            target = (target.parentNode || null);
        }
        if (eventName === '$autocapture' && e.type === 'click' && e instanceof MouseEvent) {
            if (this.instance.config.rageclick &&
                ((_a = this.rageclicks) === null || _a === void 0 ? void 0 : _a.isRageClick(e.clientX, e.clientY, new Date().getTime()))) {
                this._captureEvent(e, '$rageclick');
            }
        }
        var isCopyAutocapture = eventName === COPY_AUTOCAPTURE_EVENT;
        if (target &&
            shouldCaptureDomEvent(target, e, this.config, 
            // mostly this method cares about the target element, but in the case of copy events,
            // we want some of the work this check does without insisting on the target element's type
            isCopyAutocapture, 
            // we also don't want to restrict copy checks to clicks,
            // so we pass that knowledge in here, rather than add the logic inside the check
            isCopyAutocapture ? ['copy', 'cut'] : undefined)) {
            var targetElementList = [target];
            var curEl = target;
            while (curEl.parentNode && !isTag(curEl, 'body')) {
                if (isDocumentFragment(curEl.parentNode)) {
                    targetElementList.push(curEl.parentNode.host);
                    curEl = curEl.parentNode.host;
                    continue;
                }
                targetElementList.push(curEl.parentNode);
                curEl = curEl.parentNode;
            }
            var elementsJson_1 = [];
            var autocaptureAugmentProperties_1 = {};
            var href_1, explicitNoCapture_1 = false;
            each(targetElementList, function (el) {
                var shouldCaptureEl = shouldCaptureElement(el);
                // if the element or a parent element is an anchor tag
                // include the href as a property
                if (el.tagName.toLowerCase() === 'a') {
                    href_1 = el.getAttribute('href');
                    href_1 = shouldCaptureEl && shouldCaptureValue(href_1) && href_1;
                }
                // allow users to programmatically prevent capturing of elements by adding class 'ph-no-capture'
                var classes = getClassNames(el);
                if (includes(classes, 'ph-no-capture')) {
                    explicitNoCapture_1 = true;
                }
                elementsJson_1.push(_this._getPropertiesFromElement(el, _this.instance.config.mask_all_element_attributes, _this.instance.config.mask_all_text));
                var augmentProperties = _this._getAugmentPropertiesFromElement(el);
                extend(autocaptureAugmentProperties_1, augmentProperties);
            });
            if (!this.instance.config.mask_all_text) {
                // if the element is a button or anchor tag get the span text from any
                // children and include it as/with the text property on the parent element
                if (target.tagName.toLowerCase() === 'a' || target.tagName.toLowerCase() === 'button') {
                    elementsJson_1[0]['$el_text'] = getDirectAndNestedSpanText(target);
                }
                else {
                    elementsJson_1[0]['$el_text'] = getSafeText(target);
                }
            }
            if (href_1) {
                elementsJson_1[0]['attr__href'] = href_1;
            }
            if (explicitNoCapture_1) {
                return false;
            }
            var props = extend(this._getDefaultProperties(e.type), this._elementsChainAsString
                ? {
                    $elements_chain: getElementsChainString(elementsJson_1),
                }
                : {
                    $elements: elementsJson_1,
                }, ((_b = elementsJson_1[0]) === null || _b === void 0 ? void 0 : _b['$el_text']) ? { $el_text: (_c = elementsJson_1[0]) === null || _c === void 0 ? void 0 : _c['$el_text'] } : {}, autocaptureAugmentProperties_1);
            if (eventName === COPY_AUTOCAPTURE_EVENT) {
                // you can't read the data from the clipboard event,
                // but you can guess that you can read it from the window's current selection
                var selectedContent = makeSafeText((_d = window === null || window === void 0 ? void 0 : window.getSelection()) === null || _d === void 0 ? void 0 : _d.toString());
                var clipType = e.type || 'clipboard';
                if (!selectedContent) {
                    return false;
                }
                props['$selected_content'] = selectedContent;
                props['$copy_type'] = clipType;
            }
            this.instance.capture(eventName, props);
            return true;
        }
    };
    Autocapture.prototype.isBrowserSupported = function () {
        return isFunction(document === null || document === void 0 ? void 0 : document.querySelectorAll);
    };
    return Autocapture;
}());
export { Autocapture };
//# sourceMappingURL=autocapture.js.map